#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs = require("yargs");
const chalk_1 = __importDefault(require("chalk"));
const icona_bridge_client_1 = require("../icona-bridge-client");
const yamljs_1 = __importDefault(require("yamljs"));
const log4js_1 = __importDefault(require("log4js"));
log4js_1.default.configure({
    appenders: { 'out': { type: 'stdout' } },
    categories: { default: { appenders: ['out'], level: 'info' } }
});
const logger = log4js_1.default.getLogger('out');
logger.level = 'info';
const options = yargs
    .options({
    host: {
        description: 'Comelit HUB host',
        alias: 'h',
        type: 'string',
        demandOption: true,
    },
    port: {
        description: 'Comelit HUB port for ICONA Bridge (default ' + icona_bridge_client_1.ICONA_BRIDGE_PORT + ')',
        alias: 'p',
        type: 'number',
        demandOption: true,
        default: icona_bridge_client_1.ICONA_BRIDGE_PORT
    },
    token: {
        description: 'Icona access token',
        alias: 't',
        type: 'string',
        demandOption: true,
    },
    output: {
        description: 'Output mode: json, yaml',
        alias: 'o',
        type: 'string',
        demandOption: false,
        default: 'yaml'
    },
    debug: {
        description: 'Output debug information',
        alias: 'd',
        type: 'boolean',
        demandOption: false,
        default: false
    },
})
    .demandOption('host')
    .demandOption('token')
    .command('get-config <addressbook>', 'Get configuration of ICONA bridge', () => {
    yargs.positional('addressbook', {
        describe: 'Name of the address Book to request (valid values are none or all)',
        type: 'string',
        demandOption: true,
    });
})
    .command('server-info', 'Get server information')
    .command('list-doors', 'List all available doors using ICONA bridge')
    .command('open-door <door>', 'Open a door using ICONA bridge', () => {
    yargs.option('door', {
        describe: 'Name of the door to open',
        type: 'string',
        demandOption: true
    });
})
    .demandCommand()
    .help().argv;
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        const command = options._[0];
        if (options.debug) {
            logger.level = 'debug';
        }
        else {
            logger.level = 'info';
        }
        logger.info(chalk_1.default.green(`Executing command ${command}`));
        try {
            switch (command) {
                case 'get-config':
                    yield config();
                    break;
                case 'server-info':
                    yield serverInfo();
                    break;
                case 'push-info':
                    yield pushInfo();
                    break;
                case 'open-door':
                    yield openDoor();
                    break;
                case 'list-doors':
                    yield listDoors();
                    break;
                default:
                    logger.error(chalk_1.default.red(`Unrecognized command ${command}`));
            }
            logger.info(chalk_1.default.green('Shutting down'));
            logger.info(chalk_1.default.green(`Command ${command} executed successfully`));
        }
        catch (e) {
            logger.error(e);
        }
    });
}
function config() {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new icona_bridge_client_1.IconaBridgeClient(options.host, options.port, logger);
        yield client.connect();
        const code = yield client.authenticate(options.token);
        if (code === 200) {
            const res = yield client.getConfig(options.addressbook);
            logger.info(chalk_1.default.green(`Address books ${options.addressbook} response: `));
            logger.info(serialize(res, options.output));
            yield client.shutdown();
        }
        else {
            logger.error(chalk_1.default.red(`Error while authenticating: server responded with code ${code}`));
        }
    });
}
function serverInfo() {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new icona_bridge_client_1.IconaBridgeClient(options.host, options.port, logger);
        yield client.connect();
        const code = yield client.authenticate(options.token);
        if (code === 200) {
            const res = yield client.getServerInfo();
            logger.info(chalk_1.default.green('Server Info response: '));
            logger.info(serialize(res, options.output));
            yield client.shutdown();
        }
        else {
            logger.error(chalk_1.default.red(`Error while authenticating: server responded with code ${code}`));
        }
    });
}
function pushInfo() {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new icona_bridge_client_1.IconaBridgeClient(options.host, options.port, logger);
        yield client.connect();
        const code = yield client.authenticate(options.token);
        if (code === 200) {
            const conf = yield client.getConfig(options.addressbook);
            const res = yield client.getPushInfo(conf.vip, options.deviceToken);
            logger.info(chalk_1.default.green('Push Info response: '));
            logger.info(serialize(res, options.output));
            yield client.shutdown();
        }
        else {
            logger.error(chalk_1.default.red(`Error while authenticating: server responded with code ${code}`));
        }
    });
}
function listDoors() {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new icona_bridge_client_1.IconaBridgeClient(options.host, options.port, logger);
        yield client.connect();
        const code = yield client.authenticate(options.token);
        if (code === 200) {
            const addressBookAll = yield client.getConfig('all');
            logger.info(chalk_1.default.green(`Available doors:`));
            logger.info(serialize(addressBookAll.vip["user-parameters"]["opendoor-address-book"], options.output));
            yield client.shutdown();
        }
        else {
            logger.error(chalk_1.default.red(`Error while authenticating: server responded with code ${code}`));
        }
    });
}
function openDoor() {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new icona_bridge_client_1.IconaBridgeClient(options.host, options.port, logger);
        yield client.connect();
        try {
            const code = yield client.authenticate(options.token);
            if (code === 200) {
                const addressBook = yield client.getConfig('none', false);
                logger.info(serialize(addressBook, options.output));
                const serverInfo = yield client.getServerInfo(false);
                logger.info(serialize(serverInfo, options.output));
                const addressBookAll = yield client.getConfig('all', false);
                logger.info(serialize(addressBookAll, options.output));
                const item = addressBookAll.vip["user-parameters"]["opendoor-address-book"].find(doorItem => doorItem.name === options.door);
                if (item) {
                    logger.info(`Opening door ${item.name} at address ${item["apt-address"]} and index ${item["output-index"]}`);
                    logger.info(serialize(yield client.getServerInfo(), options.output));
                    yield client.openDoor(addressBookAll.vip, item);
                }
                else {
                    logger.error(`No door with name ${options.door} found in config. Available door names are: ${addressBookAll.vip["user-parameters"]["opendoor-address-book"].map(d => d.name).join(', ')}`);
                }
                yield client.shutdown();
            }
            else {
                logger.error(chalk_1.default.red(`Error while authenticating: server responded with code ${code}`));
            }
        }
        catch (e) {
            logger.error(chalk_1.default.red('Error while executing openDoor command'), e);
        }
        finally {
            yield client.shutdown();
        }
    });
}
function serialize(obj, output) {
    switch (output) {
        case "yaml":
            return yamljs_1.default.stringify(obj);
        case "json":
        default:
            return JSON.stringify(obj, null, 2);
    }
}
run().then(() => {
    logger.info(chalk_1.default.green('Exiting'));
    process.exit(0);
});
//# sourceMappingURL=icona-cli.js.map