"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryResponseType = exports.readJSON = exports.PacketBuilder = exports.PacketMessage = exports.Channel = exports.MessageType = exports.NO_SEQ = exports.getInt64Bytes = exports.bufferToASCIIString = exports.bufferToString = void 0;
const utils_1 = require("../utils");
function bufferToString(bytes) {
    return [...new Uint8Array(bytes)].map((x) => x.toString(16).padStart(2, '0')).join(' ');
}
exports.bufferToString = bufferToString;
function bufferToASCIIString(bytes) {
    return bytes.toString('utf-8');
}
exports.bufferToASCIIString = bufferToASCIIString;
function getInt64Bytes(x) {
    const bytes = [];
    let i = 8;
    do {
        bytes[--i] = x & (255);
        x = x >> 8;
    } while (i);
    return bytes;
}
exports.getInt64Bytes = getInt64Bytes;
const HEADER = [0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
exports.NO_SEQ = -1;
var MessageType;
(function (MessageType) {
    MessageType[MessageType["COMMAND"] = 43981] = "COMMAND";
    MessageType[MessageType["END"] = 495] = "END";
    MessageType[MessageType["OPEN_DOOR_INIT"] = 6336] = "OPEN_DOOR_INIT";
    MessageType[MessageType["OPEN_DOOR"] = 6144] = "OPEN_DOOR";
    MessageType[MessageType["OPEN_DOOR_CONFIRM"] = 6176] = "OPEN_DOOR_CONFIRM";
})(MessageType = exports.MessageType || (exports.MessageType = {}));
var Channel;
(function (Channel) {
    Channel["UAUT"] = "UAUT";
    Channel["UCFG"] = "UCFG";
    Channel["INFO"] = "INFO";
    Channel["CTPP"] = "CTPP";
    Channel["CSPB"] = "CSPB";
    Channel["PUSH"] = "PUSH";
})(Channel = exports.Channel || (exports.Channel = {}));
class PacketMessage {
    constructor(requestId, seq, bytes) {
        this.bytes = bytes;
        this.size = (bytes === null || bytes === void 0 ? void 0 : bytes.length) || 0;
        this.requestId = requestId;
        this.seq = seq;
        if (seq > 0) {
            this.type = 'binary';
        }
        else {
            this.type = 'json';
        }
    }
    dump(ascii = false) {
        if (ascii) {
            return bufferToASCIIString(this.bytes);
        }
        return bufferToString(this.bytes);
    }
    static createBinaryPacketFromBuffers(requestId, ...messages) {
        const header = Buffer.from(HEADER);
        header.writeUIntLE(requestId, 4, 2);
        const totalLength = [header, ...messages]
            .map(b => b.length)
            .reduce((s, l) => (s += l), 0);
        const buffer = Buffer.concat([header, ...messages], totalLength);
        buffer.writeUIntLE(buffer.length - HEADER.length, 2, 2);
        return new PacketMessage(requestId, exports.NO_SEQ, buffer);
    }
    static createBinaryPacketFromStrings(requestId, seq, type, ...messages) {
        const header = Buffer.from(HEADER);
        const magicNumber = Buffer.alloc(4);
        magicNumber.writeUIntLE(type, 0, 2);
        magicNumber.writeUIntLE(seq, 2, 2);
        const additionalMessages = [];
        additionalMessages.push(...messages.filter(s => s !== null && s !== undefined).map((s, i) => {
            const isFirst = i === 0;
            const info = Buffer.alloc(4);
            let text = (0, utils_1.stringToBuffer)(s);
            if (isFirst) {
                const id = Buffer.alloc(2);
                id.writeUIntLE(requestId, 0, 2);
                text = Buffer.concat([text, id, utils_1.NULL]);
                info.writeUIntLE(s.length + id.length + 1, 0, 4);
            }
            else {
                info.writeUIntLE(s.length + 1, 0, 4);
            }
            return Buffer.concat([info, text, utils_1.NULL]);
        }));
        const totalLength = [header, magicNumber, ...additionalMessages]
            .map(b => b.length)
            .reduce((s, l) => (s += l), 0);
        const buffer = Buffer.concat([header, magicNumber, ...additionalMessages], totalLength);
        buffer.writeUIntLE(buffer.length - HEADER.length, 2, 2);
        return new PacketMessage(requestId, seq, buffer);
    }
    static createJSONPacket(requestId, json) {
        const text = [...JSON.stringify(json, null, 0)].map(c => c.charCodeAt(0));
        const buffer = Buffer.from(HEADER.concat(text));
        buffer.writeUIntLE(text.length, 2, 2); // length
        buffer.writeUIntLE(requestId, 4, 2); // requestId
        return new PacketMessage(requestId, exports.NO_SEQ, buffer);
    }
}
exports.PacketMessage = PacketMessage;
class PacketBuilder {
    constructor(requestId, type, seq) {
        this.requestId = requestId;
        this.seq = seq || exports.NO_SEQ;
        const magicNumber = Buffer.alloc(4);
        magicNumber.writeUIntLE(type, 0, 2);
        if (seq) {
            magicNumber.writeUIntLE(seq, 2, 2);
        }
        let header = Buffer.from(HEADER);
        this.bytes = Buffer.concat([header, magicNumber], header.length + magicNumber.length);
    }
    build() {
        return new PacketMessage(this.requestId, this.seq, this.bytes);
    }
}
exports.PacketBuilder = PacketBuilder;
function readString(buffer) {
    const bytes = (() => {
        const chars = [];
        for (let i = 0; i < buffer.length; i++) {
            chars.push(buffer.readUIntLE(i, 1));
        }
        return chars;
    })();
    const text = String.fromCharCode(...bytes);
    return text;
}
function readJSON(buffer) {
    const text = readString(buffer);
    return JSON.parse(text);
}
exports.readJSON = readJSON;
var BinaryResponseType;
(function (BinaryResponseType) {
    BinaryResponseType[BinaryResponseType["BINARY"] = 0] = "BINARY";
    BinaryResponseType[BinaryResponseType["JSON"] = 1] = "JSON";
})(BinaryResponseType = exports.BinaryResponseType || (exports.BinaryResponseType = {}));
//# sourceMappingURL=packet.js.map